'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _ink = require('ink');

var _inkProgressBar = require('ink-progress-bar');

var _inkProgressBar2 = _interopRequireDefault(_inkProgressBar);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _inkSpinner = require('ink-spinner');

var _inkSpinner2 = _interopRequireDefault(_inkSpinner);

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

var _Conditional = require('./components/Conditional');

var _Conditional2 = _interopRequireDefault(_Conditional);

var _Container = require('./components/Container');

var _Container2 = _interopRequireDefault(_Container);

var _File = require('./util/File');

var _File2 = _interopRequireDefault(_File);

var _Zip = require('./util/Zip');

var _Zip2 = _interopRequireDefault(_Zip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const Types = {
    EXIT_WITH_SUCCESS: 0,
    EXIT_WITH_ERROR: 1
};

class Cli extends _ink.Component {
    constructor(...args) {
        var _temp;

        return _temp = super(...args), this.state = {
            error: null,
            result: null,
            projectName: null
        }, this.createScaffold = async path => {
            const gitHubFolderName = 'typescript-hapi-starter-master';

            try {
                const stream = await _Zip2.default.download('https://github.com/BlackBoxVision/typescript-hapi-starter/archive/master.zip');

                await _Zip2.default.writeStream(stream, path);

                if (await _File2.default.exists(path)) {
                    await this.handleError(`${path}.zip`, new Error(`Directory ${path} already exists. Please, retry with another <project-name>.`));
                } else {
                    await _Zip2.default.extract(path);

                    await _File2.default.remove(`${path}.zip`);
                    await _File2.default.renameFolder(gitHubFolderName, path);

                    this.setState(state => ({
                        projectName: path,
                        result: 'ok'
                    }));

                    this.exitWithDelay(Types.EXIT_WITH_SUCCESS);
                }
            } catch (error) {
                await this.handleError(gitHubFolderName, error);
            }
        }, this.handleError = async (path, error) => {
            await _File2.default.remove(path);

            this.setState(state => ({
                error: error
            }));

            this.exitWithDelay(Types.EXIT_WITH_ERROR);
        }, this.exitWithDelay = (type, delay = 1000) => setTimeout(() => process.exit(type), delay), _temp;
    }

    async componentDidMount() {
        _readline2.default.emitKeypressEvents(process.stdin);
        process.stdin.setRawMode(true);

        await this.createScaffold(this.props.projectName);
    }

    render(props, state) {
        //TODO Review
        //<Progress character="=" left={0} right={0} green />

        return (0, _ink.h)(
            _Container2.default,
            null,
            (0, _ink.h)(
                _Conditional2.default,
                { expression: !state.error && !state.result },
                (0, _ink.h)(_inkSpinner2.default, { green: true }),
                ' Fetching base project from GitHub.'
            ),
            (0, _ink.h)(
                _Conditional2.default,
                { expression: state.error !== null },
                (0, _ink.h)(
                    _ink.Text,
                    { red: true },
                    state.error && state.error.message
                )
            ),
            (0, _ink.h)(
                _Conditional2.default,
                { expression: state.result === 'ok' },
                (0, _ink.h)(
                    _ink.Text,
                    { green: true },
                    'Project ',
                    state.projectName,
                    ' has been created successfully.'
                )
            )
        );
    }

}
exports.default = Cli;
Cli.propTypes = {
    projectName: _propTypes2.default.string.isRequired
};
Cli.defaultProps = {
    projectName: 'test'
};